#pragma once

// LibAppImage Version

#define LIBAPPIMAGE_VERSION_MAJOR 1
#define LIBAPPIMAGE_VERSION_MINOR 0
#define LIBAPPIMAGE_VERSION_PATCH 3
#define LIBAPPIMAGE_VERSION_SUFFIX 

#define LIBAPPIMAGE_VERSION "1.0.3"

// Features

#define LIBAPPIMAGE_DESKTOP_INTEGRATION_ENABLED
#define LIBAPPIMAGE_THUMBNAILER_ENABLED
